@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -50,6 +50,11 @@ if (read_config_option('auth_method') == '2') {
 		auth_display_custom_error_message(__('Web Basic Authentication configured, but no username was passed from the web server. Please make sure you have authentication enabled on the web server.'));
 		exit;
 	}
+
+	if (strpos($username, '@') !== false) {
+		$upart = explode('@', $username);
+		$username = $upart[0];
+	}
 } else {
 	if (get_nfilter_request_var('action') == 'login') {
 		/* LDAP and Builtin get username from Form */
@@ -70,9 +75,9 @@ $user_enabled = 1;
 $ldap_error   = false;
 $ldap_error_message = '';
 $realm        = 0;
+$frv_realm    = get_nfilter_request_var('realm');
 
 if (get_nfilter_request_var('action') == 'login') {
-	$frv_realm = get_nfilter_request_var('realm');
 	if ($frv_realm == '1') {
 		$auth_method = 1;
 	} else {
@@ -105,10 +110,9 @@ if (get_nfilter_request_var('action') == 'login') {
 			AND realm = 2',
 			array($username));
 
-		if (!$user && read_config_option('user_template') == '0' && read_config_option('guest_user') == '0') {
+		if (!$user && get_template_account() == '0' && get_guest_account() == '0') {
 			cacti_log("ERROR: User '" . $username . "' authenticated by Web Server, but both Template and Guest Users are not defined in Cacti.  Exiting.", false, 'AUTH');
-
-			$username = htmlspecialchars($username);
+			$username = html_escape($username);
 
 			display_custom_error_message(__('%s authenticated by Web Server, but both Template and Guest Users are not defined in Cacti.', $username));
 			header('Location: index.php?header=false');
@@ -120,7 +124,7 @@ if (get_nfilter_request_var('action') == 'login') {
 		/* LDAP Auth */
  		if ($frv_realm == '2' && get_nfilter_request_var('login_password') != '') {
 			/* include LDAP lib */
-			include_once('./lib/ldap.php');
+			include_once(__DIR__ . '/lib/ldap.php');
 
 			/* get user DN */
 			$ldap_dn_search_response = cacti_ldap_search_dn($username);
@@ -158,9 +162,14 @@ if (get_nfilter_request_var('action') == 'login') {
 					$ldap_error = true;
 					$ldap_error_message = __('LDAP Error: %s', $ldap_auth_response['error_text']);
 					$user_auth = false;
+
+					$id = db_fetch_cell_prepared('SELECT id
+						FROM user_auth
+						WHERE username = ?
+						AND realm = 3',
+						array($username));
 				}
 			}
-
 		}
 
 		break;
@@ -187,22 +196,25 @@ if (get_nfilter_request_var('action') == 'login') {
 			if ((!$user_auth)) {
 				$stored_pass = db_fetch_cell_prepared('SELECT password
 					FROM user_auth
-					WHERE username = ? AND realm = 0',
+					WHERE username = ?
+					AND realm = 0',
 					array($username));
 
 				if ($stored_pass != '') {
 					$p = get_nfilter_request_var('login_password');
 					$valid = compat_password_verify($p, $stored_pass);
 
-					cacti_log("DEBUG: User '" . $username . "' password is " . ($valid?'':'in') . "valid", false, 'AUTH', POLLER_VERBOSITY_DEBUG);
+					cacti_log("DEBUG: User '" . $username . "' password is " . ($valid?'':'in') . 'valid', false, 'AUTH', POLLER_VERBOSITY_DEBUG);
 					if ($valid) {
 						$user = db_fetch_row_prepared('SELECT *
 							FROM user_auth
-							WHERE username = ? AND realm = 0',
+							WHERE username = ?
+							AND realm = 0',
 							array($username));
 
 						if (compat_password_needs_rehash($stored_pass, PASSWORD_DEFAULT)) {
 							$p = compat_password_hash($p, PASSWORD_DEFAULT);
+							db_check_password_length();
 							db_execute_prepared('UPDATE user_auth
 								SET password = ?
 								WHERE username = ?',
@@ -213,23 +225,61 @@ if (get_nfilter_request_var('action') == 'login') {
 			}
 		}
 	}
-	/* end of switch */
 
 	/* Create user from template if requested */
-	if (!sizeof($user) && $copy_user && read_config_option('user_template') != '0' && $username != '') {
-		cacti_log("WARN: User '" . $username . "' does not exist, copying template user", false, 'AUTH');
+	if (!cacti_sizeof($user) && $copy_user && get_template_account() != '0' && $username != '') {
+		cacti_log("NOTE: User '" . $username . "' does not exist, copying template user", false, 'AUTH');
+
+		$user_template = db_fetch_row_prepared('SELECT *
+			FROM user_auth
+			WHERE id = ?',
+			array(get_template_account()));
 
 		/* check that template user exists */
-		if (db_fetch_row_prepared('SELECT id FROM user_auth WHERE username = ? AND realm = 0', array(read_config_option('user_template')))) {
-			/* template user found */
-			user_copy(read_config_option('user_template'), $username, 0, $realm);
+		if (!empty($user_template)) {
+			/* get user CN*/
+			$cn_full_name = read_config_option('cn_full_name');
+			$cn_email     = read_config_option('cn_email');
+
+			if ($cn_full_name != '' || $cn_email != '') {
+				$ldap_cn_search_response = cacti_ldap_search_cn($username, array($cn_full_name, $cn_email));
+
+   				if (isset($ldap_cn_search_response['cn'])) {
+					$data_override = array();
+
+					if (array_key_exists($cn_full_name, $ldap_cn_search_response['cn'])) {
+						$data_override['full_name'] = $ldap_cn_search_response['cn'][$cn_full_name];
+					} else {
+						$data_override['full_name'] = '';
+					}
+
+					if (array_key_exists($cn_email, $ldap_cn_search_response['cn'])) {
+						$data_override['email_address'] = $ldap_cn_search_response['cn'][$cn_email];
+					} else {
+						$data_override['email_address'] = '';
+					}
+
+					user_copy($user_template['username'], $username, 0, $realm, false, $data_override);
+				} else {
+					$ldap_response = (isset($ldap_cn_search_response[0]) ? $ldap_cn_search_response[0] : '(no response given)');
+					$ldap_code = (isset($ldap_cn_search_response['error_num']) ? $ldap_cn_search_response['error_num'] : '(no code given)');
+					cacti_log('LOGIN: Email Address and Full Name fields not found, reason: ' . $ldap_response . 'code: ' . $ldap_code, false, 'AUTH');
+					user_copy($user_template['username'], $username, 0, $realm);
+				}
+			} else {
+				user_copy($user_template['username'], $username, 0, $realm);
+			}
 
 			/* requery newly created user */
-			$user = db_fetch_row_prepared('SELECT * FROM user_auth WHERE username = ? AND realm = ?', array($username, $realm));
+			$user = db_fetch_row_prepared('SELECT *
+				FROM user_auth
+				WHERE username = ?
+				AND realm = ?',
+				array($username, $realm));
 		} else {
 			/* error */
-			display_custom_error_message(__('Template user %s does not exist.', read_config_option('user_template')));
-			cacti_log("LOGIN: Template user '" . read_config_option('user_template') . "' does not exist.", false, 'AUTH');
+			display_custom_error_message(__('Template user id %s does not exist.', read_config_option('user_template')));
+			cacti_log("LOGIN: Template user id '" . read_config_option('user_template') . "' does not exist.", false, 'AUTH');
 			header('Location: index.php?header=false');
 			exit;
 		}
@@ -237,12 +287,12 @@ if (get_nfilter_request_var('action') == 'login') {
 
 	/* Guest account checking - Not for builtin */
 	$guest_user = false;
-	if ((!sizeof($user)) && ($user_auth) && (read_config_option('guest_user') != '0')) {
+	if (!cacti_sizeof($user) && $user_auth && get_guest_account() != '0') {
 		/* Locate guest user record */
-		$user = db_fetch_row_prepared('SELECT *
+		$user = db_fetch_row_prepared('SELECT id, username, enabled
 			FROM user_auth
-			WHERE username = ?',
-			array(read_config_option('guest_user')));
+			WHERE id = ?',
+			array(get_guest_account()));
 
 		if ($user) {
 			cacti_log("LOGIN: Authenticated user '" . $username . "' using guest account '" . $user['username'] . "'", false, 'AUTH');
@@ -250,7 +300,7 @@ if (get_nfilter_request_var('action') == 'login') {
 			$guest_user = true;
 		} else {
 			/* error */
-			display_custom_error_message(__('Guest user %s does not exist.', read_config_option('guest_user')));
+			display_custom_error_message(__('Guest user id %s does not exist.', read_config_option('guest_user')));
 			cacti_log("LOGIN: Unable to locate guest user '" . read_config_option('guest_user') . "'", false, 'AUTH');
 			header('Location: index.php?header=false');
 			exit;
@@ -258,24 +308,10 @@ if (get_nfilter_request_var('action') == 'login') {
 	}
 
 	/* Process the user  */
-	if (sizeof($user)) {
+	if (cacti_sizeof($user)) {
 		cacti_log("LOGIN: User '" . $user['username'] . "' Authenticated", false, 'AUTH');
 
-		if (isset($_SERVER['HTTP_CLIENT_IP'])) {
-			$client_addr = $_SERVER['HTTP_CLIENT_IP'];
-		} elseif (isset($_SERVER['X-Forwarded-For'])) {
-			$client_addr = $_SERVER['X-Forwarded-For'];
-		} elseif (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
-			$client_addr = $_SERVER['HTTP_X_FORWARDED_FOR'];
-		} elseif (isset($_SERVER['HTTP_FORWARDED_FOR'])) {
-			$client_addr = $_SERVER['HTTP_FORWARDED_FOR'];
-		} elseif (isset($_SERVER['HTTP_FORWARDED'])) {
-			$client_addr = $_SERVER['HTTP_FORWARDED'];
-		} elseif (isset($_SERVER['REMOTE_ADDR'])) {
-			$client_addr = $_SERVER['REMOTE_ADDR'];
-		} else {
-			$client_addr = '';
-		}
+		$client_addr = get_client_addr('');
 
 		db_execute_prepared('INSERT IGNORE INTO user_log
 			(username, user_id, result, ip, time)
@@ -300,21 +336,21 @@ if (get_nfilter_request_var('action') == 'login') {
 		$_SESSION['sess_user_id'] = $user['id'];
 
 		/* handle 'force change password' */
-		if (($user['must_change_password'] == 'on') &&
-			(read_config_option('auth_method') == 1) &&
-			($user['password_change'] == 'on')) {
-
+		if ($user['must_change_password'] == 'on' && read_config_option('auth_method') == 1 && $user['password_change'] == 'on') {
 			$_SESSION['sess_change_password'] = true;
 		}
 
-		$group_options = db_fetch_cell_prepared('SELECT MAX(login_opts)
-			FROM user_auth_group AS uag
-			INNER JOIN user_auth_group_members AS uagm
-			ON uag.id=uagm.group_id
-			WHERE user_id=?', array($_SESSION['sess_user_id']));
+		if (db_table_exists('user_auth_group')) {
+			$group_options = db_fetch_cell_prepared('SELECT MAX(login_opts)
+				FROM user_auth_group AS uag
+				INNER JOIN user_auth_group_members AS uagm
+				ON uag.id=uagm.group_id
+				WHERE user_id=?',
+				array($_SESSION['sess_user_id']));
 
-		if ($group_options > 0) {
-			$user['login_opts'] = $group_options;
+			if ($group_options > 0) {
+				$user['login_opts'] = $group_options;
+			}
 		}
 
 		$newtheme = false;
@@ -323,6 +359,10 @@ if (get_nfilter_request_var('action') == 'login') {
 			$newtheme = true;
 		}
 
+		if (user_setting_exists('user_language', $_SESSION['sess_user_id'])) {
+			$_SESSION['sess_user_language'] = read_user_setting('user_language');
+		}
+
 		/* ok, at the point the user has been sucessfully authenticated; so we must
 		decide what to do next */
 		switch ($user['login_opts']) {
@@ -373,21 +413,34 @@ if (get_nfilter_request_var('action') == 'login') {
 			default:
 				api_plugin_hook_function('login_options_navigate', $user['login_opts']);
 		}
+
+		exit;
+	} elseif (!$guest_user && $user_auth) {
+		/* No guest account defined */
+		display_custom_error_message(__('Access Denied, please contact you Cacti Administrator.'));
+		cacti_log('LOGIN: Access Denied, No guest enabled or template user to copy', false, 'AUTH');
+		header('Location: index.php');
 		exit;
 	} else {
-		if ((!$guest_user) && ($user_auth)) {
-			/* No guest account defined */
-			display_custom_error_message(__('Access Denied, please contact you Cacti Administrator.'));
-			cacti_log('LOGIN: Access Denied, No guest enabled or template user to copy', false, 'AUTH');
-			header('Location: index.php');
-			exit;
+		if ($auth_method == 1) {
+			$realm = 0;
 		} else {
-			/* BAD username/password builtin and LDAP */
-			db_execute_prepared('INSERT IGNORE INTO user_log
-				(username, user_id, result, ip, time)
-				VALUES (?, 0, 0, ?, NOW())',
-				array($username, $_SERVER['REMOTE_ADDR']));
+			$realm = 3;
 		}
+
+		$id = db_fetch_cell_prepared('SELECT id
+			FROM user_auth
+			WHERE username = ?
+			AND realm = ?',
+			array($username, $realm));
+
+		/* BAD username/password builtin and LDAP */
+		db_execute_prepared('INSERT IGNORE INTO user_log
+			(username, user_id, result, ip, time)
+			VALUES (?, ?, 0, ?, NOW())',
+			array($username, !empty($id) ? $id:0, get_client_addr('')));
+
+		cacti_log('LOGIN: ' . ($realm == 0 ? 'Local':'LDAP') . " Login Failed for user '" . $username . "' from IP Address '" . get_client_addr('') . "'.", false, 'AUTH');
 	}
 }
 
@@ -417,7 +470,7 @@ function domains_login_process() {
 
 	if (is_numeric(get_nfilter_request_var('realm')) && get_nfilter_request_var('login_password') != '') {
 		/* include LDAP lib */
-		include_once('./lib/ldap.php');
+		include_once(__DIR__ . '/lib/ldap.php');
 
 		/* get user DN */
 		$ldap_dn_search_response = domains_ldap_search_dn($username, get_nfilter_request_var('realm'));
@@ -466,12 +519,53 @@ function domains_login_process() {
 					WHERE id = ?',
 					array($template_user));
 
-				if (!sizeof($user) && $copy_user && !empty($template_user) && $username != '') {
+				if (!cacti_sizeof($user) && $copy_user && $template_user > 0 && $username != '') {
 					cacti_log("WARN: User '" . $username . "' does not exist, copying template user", false, 'AUTH');
+
 					/* check that template user exists */
-					if (db_fetch_row_prepared('SELECT id FROM user_auth WHERE id = ? AND realm = 0', array($template_user))) {
+					$user_template = db_fetch_row_prepared('SELECT *
+						FROM user_auth
+						WHERE id = ?',
+						array(get_template_account()));
+
+					if (!empty($user_template['id']) && $user_template['id'] > 0) {
 						/* template user found */
-						user_copy($template_username, $username, 0, $realm);
+						$cn_full_name = db_fetch_cell_prepared('SELECT cn_full_name
+							FROM user_domains_ldap
+							WHERE domain_id = ?',
+							array(get_nfilter_request_var('realm')-1000));
+
+						$cn_email = db_fetch_cell_prepared('SELECT cn_email
+							FROM user_domains_ldap
+							WHERE domain_id = ?',
+							array(get_nfilter_request_var('realm')-1000));
+
+						if ($cn_full_name != '' || $cn_email != '') {
+							$ldap_cn_search_response = cacti_ldap_search_cn($username, array($cn_full_name,$cn_email) );
+
+							if (isset($ldap_cn_search_response['cn'])) {
+								$data_override = array();
+
+								if (array_key_exists($cn_full_name, $ldap_cn_search_response['cn'])) {
+									$data_override['full_name'] = $ldap_cn_search_response['cn'][$cn_full_name];
+								} else {
+									$data_override['full_name'] = '';
+								}
+
+								if (array_key_exists($cn_email, $ldap_cn_search_response['cn'])) {
+									$data_override['email_address'] = $ldap_cn_search_response['cn'][$cn_email];
+								} else {
+									$data_override['email_address'] = '';
+								}
+
+								user_copy($user_template['username'], $username, 0, $realm, false, $data_override);
+							} else {
+								cacti_log('LOGIN: fields not found ' . $ldap_cn_search_response[0] . 'code: ' . $ldap_cn_search_response['error_num'], false, 'AUTH');
+								user_copy($user_template['username'], $username, 0, $realm);
+							}
+						} else {
+							user_copy($user_template['username'], $username, 0, $realm);
+						}
 
 						/* requery newly created user */
 						$user = db_fetch_row_prepared('SELECT *
@@ -481,8 +575,8 @@ function domains_login_process() {
 							array($username, $realm));
 					} else {
 						/* error */
-						display_custom_error_message(__('Template user %s does not exist.', $template_username));
-						cacti_log("LOGIN: Template user '" . $template_username . "' does not exist.", false, 'AUTH');
+						display_custom_error_message(__('Template user id %s does not exist.', $template_id));
+						cacti_log("LOGIN: Template user id '" . $template_id . "' does not exist.", false, 'AUTH');
 						header('Location: index.php?header=false');
 						exit;
 					}
@@ -511,7 +605,7 @@ function domains_ldap_auth($username, $password = '', $dn = '', $realm) {
 		WHERE domain_id = ?',
 		array($realm-1000));
 
-	if (sizeof($ld)) {
+	if (cacti_sizeof($ld)) {
 		if (!empty($ld['dn']))                $ldap->dn                = $ld['dn'];
 		if (!empty($ld['server']))            $ldap->host              = $ld['server'];
 		if (!empty($ld['port']))              $ldap->port              = $ld['port'];
@@ -551,7 +645,7 @@ function domains_ldap_search_dn($username, $realm) {
 		WHERE domain_id = ?',
 		array($realm-1000));
 
-	if (sizeof($ld)) {
+	if (cacti_sizeof($ld)) {
 		if (!empty($ld['dn']))                $ldap->dn                = $ld['dn'];
 		if (!empty($ld['server']))            $ldap->host              = $ld['server'];
 		if (!empty($ld['port']))              $ldap->port              = $ld['port'];
@@ -593,11 +687,6 @@ $selectedTheme = get_selected_theme();
 <html>
 <head>
 	<?php html_common_header(api_plugin_hook_function('login_title', __('Login to Cacti')));?>
-	<script type='text/javascript'>
-	$(function() {
-			$('#login_username').focus();
-	});
-	</script>
 </head>
 <body class='loginBody'>
 	<div class='loginLeft'></div>
@@ -605,7 +694,7 @@ $selectedTheme = get_selected_theme();
 	<div class='loginArea'>
 		<div class='cactiLoginLogo'></div>
 			<legend><?php print __('User Login');?></legend>
-			<form name='login' method='post' action='<?php print get_current_page();?>'>
+			<form id='login' name='login' method='post' action='<?php print get_current_page();?>'>
 				<input type='hidden' name='action' value='login'>
 				<?php api_plugin_hook_function('login_before',
 					array(
@@ -625,7 +714,7 @@ $selectedTheme = get_selected_theme();
 								<label for='login_username'><?php print __('Username');?></label>
 							</td>
 							<td>
-								<input type='text' id='login_username' name='login_username' value='<?php print htmlspecialchars($username, ENT_QUOTES); ?>' placeholder='<?php print __esc('Username');?>'>
+								<input type='text' class='ui-state-default ui-corner-all' id='login_username' name='login_username' value='<?php print html_escape($username); ?>' placeholder='<?php print __esc('Username');?>'>
 							</td>
 						</tr>
 						<tr>
@@ -633,7 +722,7 @@ $selectedTheme = get_selected_theme();
 								<label for='login_password'><?php print __('Password');?></label>
 							</td>
 							<td>
-								<input type='password' id='login_password' name='login_password' placeholder='********'>
+								<input type='password' autocomplete='new-password' class='ui-state-default ui-corner-all' id='login_password' name='login_password' placeholder='********'>
 							</td>
 						</tr>
 						<?php
@@ -654,16 +743,23 @@ $selectedTheme = get_selected_theme();
 							} else {
 								$realms = get_auth_realms(true);
 							}
+
+							// try and remember previously selected realm
+							if ($frv_realm && array_key_exists($frv_realm, $realms)) {
+								foreach ($realms as $key => $realm) {
+									$realms[$key]['selected'] = ($frv_realm == $key);
+								}
+							}
 						?>
 						<tr>
 							<td>
 								<label for='realm'><?php print __('Realm');?></label>
 							</td>
 							<td>
-								<select id='realm' name='realm'><?php
-									if (sizeof($realms)) {
-										foreach ($realms as $index => $realm) {
-											print "\t\t\t\t\t<option value='" . $index . "'" . ($realm['selected'] ? ' selected':'') . '>' . htmlspecialchars($realm['name']) . "</option>\n";
+								<select id='realm' name='realm' class='ui-state-default ui-corner-all'><?php
+									if (cacti_sizeof($realms)) {
+										foreach($realms as $index => $realm) {
+											print "\t\t\t\t\t<option value='" . $index . "'" . ($realm['selected'] ? ' selected="selected"':'') . '>' . html_escape($realm['name']) . "</option>\n";
 										}
 									}
 									?>
@@ -673,14 +769,14 @@ $selectedTheme = get_selected_theme();
 					<?php } if (read_config_option('auth_cache_enabled') == 'on') { ?>
 						<tr>
 							<td colspan='2'>
-								<input style='vertical-align:-3px;' type='checkbox' id='remember_me' name='remember_me' <?php print (isset($_COOKIE['cacti_remembers']) ? 'checked':'');?>>
+								<input style='vertical-align:-3px;' type='checkbox' id='remember_me' name='remember_me' <?php print (isset($_COOKIE['cacti_remembers']) || !isempty_request_var('remember_me') ? 'checked':'');?>>
 								<label for='remember_me'><?php print __('Keep me signed in');?></label>
 							</td>
 						</tr>
 					<?php } ?>
 						<tr>
 							<td cospan='2'>
-								<input type='submit' value='<?php print __esc('Login');?>'>
+								<input type='submit' class='ui-button ui-corner-all ui-widget' value='<?php print __esc('Login');?>'>
 							</td>
 						</tr>
 					</table>
@@ -706,12 +802,40 @@ $selectedTheme = get_selected_theme();
 	</div>
 	<div class='loginRight'></div>
 	<script type='text/javascript'>
+	var storage = Storages.localStorage;
+
 	$(function() {
+		preferredRealm = storage.get('user_realm');
+
+		// Restore the preferred realm
+		if ($('#realm').length) {
+			if (preferredRealm !== undefined) {
+				$('#realm').val(preferredRealm);
+				if ($('#realm').selectmenu('instance') !== undefined) {
+					$('#realm').selectmenu('refresh');
+				}
+			}
+		}
+
+		// Control submit in order to store preferred realm
+		$('#login').submit(function(event) {
+			event.preventDefault();
+			if ($('#realm').length) {
+				storage.set('user_realm', $('#realm').val());
+			}
+			$('#login').off('submit').trigger('submit');
+		});
+
 		$('body').css('height', $(window).height());
 		$('.loginLeft').css('width',parseInt($(window).width()*0.33)+'px');
 		$('.loginRight').css('width',parseInt($(window).width()*0.33)+'px');
+<?php if (empty($username)) { ?>
+		$('#login_username').focus();
+<?php } else { ?>
+		$('#login_password').focus();
+<?php } ?>
 	});
 	</script>
-	<?php include_once('./include/global_session.php');?>
+	<?php include_once(__DIR__ . '/include/global_session.php');?>
 </body>
 </html>
